#!/usr/bin/python3

# Copyright (C) 2015  Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


import unittest

import os
import sys
import tempfile
import shutil

DIR = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(DIR, ".."))

from productmd.discinfo import DiscInfo  # noqa


class TestDiscInfo(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.discinfo_path = os.path.join(DIR, "discinfo")

    def setUp(self):
        self.maxDiff = None
        self.tmp_dir = tempfile.mkdtemp()
        self.path = os.path.join(self.tmp_dir, "discinfo")

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_empty(self):
        di = DiscInfo()
        self.assertRaises(ValueError, di.dump, self.path)

    def test_invalid_values(self):
        di = DiscInfo()
        di.timestamp = "foo"
        di.description = "Fedora 21"
        di.arch = "x86_64"
        di.disc_numbers = ["ALL"]
        self.assertRaises(TypeError, di.dumps)

    def assertSameFiles(self, path1, path2):
        self.assertEqual(os.path.getsize(path1), os.path.getsize(path2))
        with open(path1, "r") as file1:
            with open(path2, "r") as file2:
                self.assertEqual(file1.read(), file2.read())

    def _test_identity(self, di):
        first = os.path.join(self.tmp_dir, "first")
        second = os.path.join(self.tmp_dir, "second")

        # write original file
        di.dump(first)

        # read file and write it back
        di = DiscInfo()
        di.load(first)
        di.dump(second)

        # check if first and second files are identical
        self.assertSameFiles(first, second)

    def test_all(self):
        di = DiscInfo()
        di.now()
        di.description = "Fedora 20"
        di.arch = "x86_64"
        di.disc_numbers = ["ALL"]
        di.dump(self.path)

        self._test_identity(di)

    def test_disc_numbers(self):
        di = DiscInfo()
        di.timestamp = 1386856788.124593
        di.description = "Fedora 20"
        di.arch = "x86_64"
        di.disc_numbers = [1, "2", 3]
        di.dump(self.path)

        self._test_identity(di)

    def test_parse(self):
        lines = [
            "1386856788.124593",
            "Fedora 20",
            "x86_64",
            "ALL",
        ]
        di = DiscInfo()
        di.loads("\n".join(lines))
        self.assertEqual(di.timestamp, 1386856788.124593)
        self.assertEqual(di.description, "Fedora 20")
        self.assertEqual(di.arch, "x86_64")
        self.assertEqual(di.disc_numbers, ["ALL"])

    def test_parse_disc_numbers(self):
        lines = [
            "1386856788.124593",
            "Fedora 20",
            "x86_64",
            "1,2, 3,4 ",
        ]
        di = DiscInfo()
        di.loads("\n".join(lines))
        self.assertEqual(di.timestamp, 1386856788.124593)
        self.assertEqual(di.description, "Fedora 20")
        self.assertEqual(di.arch, "x86_64")
        self.assertEqual(di.disc_numbers, [1, 2, 3, 4])

    def test_parse_no_disc_numbers(self):
        lines = [
            "1386856788.124593",
            "Fedora 20",
            "x86_64",
        ]
        di = DiscInfo()
        di.loads("\n".join(lines))
        self.assertEqual(di.timestamp, 1386856788.124593)
        self.assertEqual(di.description, "Fedora 20")
        self.assertEqual(di.arch, "x86_64")
        self.assertEqual(di.disc_numbers, ["ALL"])

    def test_read_discinfo(self):
        for i in os.listdir(self.discinfo_path):
            path = os.path.join(self.discinfo_path, i)
            di = DiscInfo()
            di.load(path)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2019  Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, see <https://gnu.org/licenses/>.

import json

import productmd.common
from productmd.common import Header, RPM_ARCHES
from productmd.composeinfo import Compose

__all__ = (
    "ExtraFiles",
)


class ExtraFiles(productmd.common.MetadataBase):
    def __init__(self):
        super().__init__()
        self.header = Header(self, "productmd.extra_files")
        self.compose = Compose(self)
        self.extra_files = {}

    def __getitem__(self, variant):
        return self.extra_files[variant]

    def __delitem__(self, variant):
        del self.extra_files[variant]

    def serialize(self, parser):
        self.validate()
        data = parser
        self.header.serialize(data)
        data["payload"] = {}
        self.compose.serialize(data["payload"])
        data["payload"]["extra_files"] = self.extra_files
        return data

    def deserialize(self, data):
        self.header.deserialize(data)
        self.compose.deserialize(data["payload"])
        self.extra_files = data["payload"]["extra_files"]
        self.validate()

    def add(self, variant, arch, path, size, checksums):
        if not variant:
            raise ValueError("Non-empty variant is expected")

        if arch not in RPM_ARCHES:
            raise ValueError("Arch not found in RPM_ARCHES: %s" % arch)

        if not path:
            raise ValueError("Path can not be empty.")

        if path.startswith("/"):
            raise ValueError("Relative path expected: %s" % path)

        if not isinstance(checksums, dict):
            raise TypeError("Checksums must be a dict.")

        metadata = self.extra_files.setdefault(variant, {}).setdefault(arch, [])
        metadata.append({"file": path, "size": size, "checksums": checksums})

    def dump_for_tree(self, output, variant, arch, basepath):
        """Dump the serialized metadata for given tree. The basepath is
        stripped from all paths.
        """
        metadata = {"header": {"version": "1.0"}, "data": []}
        for item in self.extra_files[variant][arch]:
            metadata["data"].append(
                {
                    "file": _relative_to(item["file"], basepath),
                    "size": item["size"],
                    "checksums": item["checksums"],
                }
            )

        json.dump(metadata, output, sort_keys=True, indent=4, separators=(",", ": "))


def _relative_to(path, root):
    root = root.rstrip("/") + "/"
    if path.startswith(root):
        return path[len(root):]
    return path



# Copyright (C) 2015  Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


"""
This module provides classes for manipulating images.json files.
images.json files provide details about images included in composes.

Example::
  import productmd.compose
  compose = productmd.compose.Compose("/path/to/compose")

  # Print the entire dict that maps all variants, arches, and images for this
  # compose:
  print(compose.images.images)

  # Find all the qcow2 images in this compose:
  qcow2s = set()

  for variant in compose.images.images:
      for arch in compose.images.images[variant]:
          for images in compose.images.images[variant].values():
              if image.type == 'qcow2':
                  qcow2s.add(image)

  print(qcow2s)

  # ... prints the set of qcow2 images in all our variants:
  [<Image:Server-RT/x86_64/images/rhel-kvm-rt-image-7.6-220.x86_64.qcow2:qcow2:x86_64>,
 <Image:Server/x86_64/images/rhel-guest-image-7.6-210.x86_64.qcow2:qcow2:x86_64>,
 <Image:Server/ppc64le/images/rhel-guest-image-7.6-210.ppc64le.qcow2:qcow2:ppc64le>]

"""


import productmd.common
from productmd.common import Header
from productmd.composeinfo import Compose

from collections import namedtuple
from itertools import chain

__all__ = (
    "Image",
    "Images",
    "IMAGE_TYPE_FORMAT_MAPPING"
    "SUPPORTED_IMAGE_TYPES",
    "SUPPORTED_IMAGE_FORMATS",
    "UNIQUE_IMAGE_ATTRIBUTES",
    "UniqueImage",
)

IMAGE_TYPE_FORMAT_MAPPING = {
    'appx': ['appx'],
    'boot': ['iso'],
    # for all bootable container images, see https://github.com/containers/bootc
    # and https://coreos.github.io/rpm-ostree/container/
    'bootable-container': ['ociarchive'],
    'cd': ['iso'],
    'container': ['tar.xz', 'oci', 'ociarchive'],
    'docker': ['tar.gz', 'tar.xz'],
    'dvd': ['iso'],
    # Usually non-bootable image which contains a repo with debuginfo packages.
    'dvd-debuginfo': ['iso'],
    # installer image that deploys a payload containing an ostree-based
    # distribution
    'dvd-ostree': ['iso'],
    'dvd-ostree-osbuild': ['iso'],
    'ec2': [],
    # these back FEX:
    # https://fedoraproject.org/wiki/Changes/FEX
    'fex': ['erofs.xz', 'erofs.gz', 'erofs', 'squashfs.xz', 'squashfs.gz', 'squashfs'],
    'kvm': [],
    'live': [],
    'live-osbuild': ['iso'],
    'liveimg-squashfs': ['liveimg.squashfs'],
    'netinst': ['iso'],
    'ociarchive': ['ociarchive'],
    'p2v': [],
    'qcow': ['qcow'],
    'qcow2': ['qcow2'],
    'raw': ['raw'],
    'raw-xz': ['raw.xz'],
    'rescue': [],
    'rhevm-ova': ['rhevm.ova'],
    # raw disk image named `disk.raw` stuffed into a gzipped tarball
    # format required for import by Google Compute Engine:
    # https://cloud.google.com/compute/docs/images/import-existing-image
    'tar-gz': ['tar.gz'],
    'vagrant-hyperv': ['vagrant-hyperv.box'],
    'vagrant-libvirt': ['vagrant-libvirt.box'],
    'vagrant-virtualbox': ['vagrant-virtualbox.box'],
    'vagrant-vmware-fusion': ['vagrant-vmware-fusion.box'],
    'vdi': ['vdi'],
    'vmdk': ['vmdk'],
    'vpc': ['vhd'],
    'vhd-compressed': ['vhd.gz', 'vhd.xz'],
    'vsphere-ova': ['vsphere.ova'],
    # https://learn.microsoft.com/en-us/windows/wsl/use-custom-distro
    'wsl2': ['tar', 'tar.gz', 'wsl'],
}

#: supported image types
SUPPORTED_IMAGE_TYPES = list(sorted(IMAGE_TYPE_FORMAT_MAPPING.keys()))

#: supported image formats, they match with file suffix
SUPPORTED_IMAGE_FORMATS = list(sorted(set(chain(*IMAGE_TYPE_FORMAT_MAPPING.values()))))

#: combination of attributes which uniquely identifies an image across composes
UNIQUE_IMAGE_ATTRIBUTES = [
    "subvariant",
    "type",
    "format",
    "arch",
    "disc_number",
    "unified",
    "additional_variants",
]
#: a namedtuple with unique attributes, use ``identify_image`` to create an instance
UniqueImage = namedtuple('UniqueImage', UNIQUE_IMAGE_ATTRIBUTES)


class Images(productmd.common.MetadataBase):
    def __init__(self):
        super().__init__()
        self.header = Header(self, "productmd.images")
        self.compose = Compose(self)
        self.images = {}

    def __getitem__(self, variant):
        return self.images[variant]

    def __delitem__(self, variant):
        del self.images[variant]

    def serialize(self, parser):
        data = parser
        self.header.serialize(data)
        data["payload"] = {}
        data["payload"]["images"] = {}
        self.compose.serialize(data["payload"])
        for variant in self.images:
            for arch in self.images[variant]:
                for image_obj in self.images[variant][arch]:
                    images = data["payload"]["images"].setdefault(variant, {}).setdefault(arch, [])
                    image_obj.serialize(images)
                    images.sort(key=lambda x: x["path"])
        return data

    def deserialize(self, data):
        self.header.deserialize(data)
        self.compose.deserialize(data["payload"])
        for variant in data["payload"]["images"]:
            for arch in data["payload"]["images"][variant]:
                for image in data["payload"]["images"][variant][arch]:
                    image_obj = Image(self)
                    image_obj.deserialize(image)
                    if self.header.version_tuple <= (1, 1):
                        self._add_1_1(data, variant, arch, image_obj)
                    else:
                        self.add(variant, arch, image_obj)
        self.header.set_current_version()

    def _add_1_1(self, data, variant, arch, image):
        if arch == "src":
            # move src under binary arches
            for variant_arch in data["payload"]["images"][variant]:
                if variant_arch == "src":
                    continue
                self.add(variant, variant_arch, image)
        else:
            self.add(variant, arch, image)

    def add(self, variant, arch, image):
        """
        Assign an :class:`.Image` object to variant and arch.

        :param variant: compose variant UID
        :type  variant: str
        :param arch:    compose architecture
        :type  arch:    str
        :param image:   image
        :type  image:   :class:`.Image`
        """

        if arch not in productmd.common.RPM_ARCHES:
            raise ValueError("Arch not found in RPM_ARCHES: %s" % arch)
        if arch in ["src", "nosrc"]:
            raise ValueError("Source arch is not allowed. Map source files under binary arches.")
        if self.header.version_tuple >= (1, 1):
            # disallow adding a different image with same 'unique'
            # attributes. can't do this pre-1.1 as we couldn't truly
            # identify images before subvariant
            for checkvar in self.images:
                for checkarch in self.images[checkvar]:
                    for curimg in self.images[checkvar][checkarch]:
                        if identify_image(curimg) == identify_image(image) and curimg.checksums != image.checksums:
                            raise ValueError("Image {0} shares all UNIQUE_IMAGE_ATTRIBUTES with "
                                             "image {1}! This is forbidden.".format(image, curimg))
        self.images.setdefault(variant, {}).setdefault(arch, set()).add(image)


def identify_image(image):
    """Provides a tuple of image's UNIQUE_IMAGE_ATTRIBUTES. Note:
    this is not guaranteed to be unique (and will often not be)
    for pre-1.1 metadata, as subvariant did not exist. Provided as
    a function so consumers can use it on plain image dicts read from
    the metadata or PDC.
    """
    try:
        # Image instance case
        attrs = tuple(getattr(image, attr) for attr in UNIQUE_IMAGE_ATTRIBUTES)
    except AttributeError:
        # Plain dict case
        attrs = tuple(image.get(attr, None) for attr in UNIQUE_IMAGE_ATTRIBUTES)
    ui = UniqueImage(*attrs)
    # If unified is None (which could happen in the dict case, we want default
    # value of False instead. Also convert additional_variants to a list.
    return ui._replace(
        unified=ui.unified or False, additional_variants=ui.additional_variants or []
    )


class Image(productmd.common.MetadataBase):
    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.path = None                #: (*str*) -- relative path to an image, for example: "Server/x86_64/iso/boot.iso"
        self.mtime = None               #: (*int*) -- image mtime
        self.size = None                #: (*int*) -- image size
        self.volume_id = None           #: (*str*) --
        self.type = None                #: (*str*) --
        self.format = None              #: (*str*) -- Release name, for example: "Fedora", "Red Hat Enterprise Linux"
        self.arch = None                #: (*str*) -- image architecture, for example: "x86_64", "src"
        self.disc_number = None         #: (*int*) -- Release name, for example: "Fedora", "Red Hat Enterprise Linux"
        self.disc_count = None          #: (*int*) -- Release name, for example: "Fedora", "Red Hat Enterprise Linux"
        self.checksums = {}             #: (*str*) -- Release name, for example: "Fedora", "Red Hat Enterprise Linux"
        self.implant_md5 = None         #: (*str* or *None*) -- value of implanted md5
        self.bootable = False           #: (*bool=False*) --
        self.subvariant = None          #: (*str*) -- image contents, may be same as variant or e.g. 'KDE', 'LXDE'
        self.unified = False            #: (*bool=False*) -- indicates if the ISO contains content from multiple variants
        self.additional_variants = []   #: (*[str]*) -- indicates which variants are present on the ISO

    def __repr__(self):
        return "<Image:{0.path}:{0.format}:{0.arch}>".format(self)

    def _validate_path(self):
        self._assert_type("path", [str])
        self._assert_not_blank("path")

    def _validate_mtime(self):
        self._assert_type("mtime", [int])

    def _validate_size(self):
        self._assert_type("size", [int])
        self._assert_not_blank("size")

    def _validate_volume_id(self):
        self._assert_type("volume_id", [type(None), str])
        if self.volume_id is not None:
            self._assert_not_blank("volume_id")

    def _validate_type(self):
        self._assert_type("type", [str])
        self._assert_value("type", SUPPORTED_IMAGE_TYPES)

    def _validate_format(self):
        self._assert_type("format", [str])
        self._assert_value("format", SUPPORTED_IMAGE_FORMATS)

    def _validate_arch(self):
        self._assert_type("arch", [str])
        self._assert_not_blank("arch")

    def _validate_disc_number(self):
        self._assert_type("disc_number", [int])

    def _validate_disc_count(self):
        self._assert_type("disc_count", [int])

    def _validate_checksums(self):
        self._assert_type("checksums", [dict])
        self._assert_not_blank("checksums")

    def _validate_implant_md5(self):
        self._assert_type("implant_md5", [type(None), str])
        if self.implant_md5 is not None:
            self._assert_matches_re("implant_md5", [r"^[a-z0-9]{32}$"])

    def _validate_bootable(self):
        self._assert_type("bootable", [bool])

    def _validate_subvariant(self):
        self._assert_type("subvariant", [str])

    def _validate_unified(self):
        self._assert_type("unified", [bool])

    def _validate_merges_variants(self):
        self._assert_type("additional_variants", [list])
        if self.additional_variants and not self.unified:
            raise ValueError("Only unified images can contain multiple variants")

    def serialize(self, parser):
        data = parser
        self.validate()
        result = {
            "path": self.path,
            "mtime": self.mtime,
            "size": self.size,
            "volume_id": self.volume_id,
            "type": self.type,
            "format": self.format,
            "arch": self.arch,
            "disc_number": self.disc_number,
            "disc_count": self.disc_count,
            "checksums": self.checksums,
            "implant_md5": self.implant_md5,
            "bootable": self.bootable,
            "subvariant": self.subvariant,
        }
        if self.unified:
            # Only add the `unified` field if it doesn't have the default value.
            result['unified'] = self.unified
            result["additional_variants"] = self.additional_variants
        data.append(result)

    def deserialize(self, data):
        self.path = data["path"]
        self.mtime = int(data["mtime"])
        self.size = int(data["size"])
        self.volume_id = data["volume_id"]
        self.type = data["type"]
        self.format = data.get("format", "iso")
        self.arch = data["arch"]
        self.disc_number = int(data["disc_number"])
        self.disc_count = int(data["disc_count"])
        self.checksums = data["checksums"]
        self.implant_md5 = data["implant_md5"]
        self.bootable = bool(data["bootable"])
        if self.parent.header.version_tuple <= (1, 0):
            self.subvariant = data.get("subvariant", "")
        else:
            # 1.1+
            self.subvariant = data["subvariant"]
        self.unified = data.get('unified', False)
        self.additional_variants = data.get("additional_variants", [])
        self.validate()

    def add_checksum(self, root, checksum_type, checksum_value):
        if checksum_type in self.checksums:
            if checksum_value and checksum_value != self.checksums[checksum_type]:
                raise ValueError("Existing and added checksums do not match: %s vs %s" % (self.checksums[checksum_type], checksum_value))
            return self.checksums[checksum_type]

        self.checksums[checksum_type] = checksum_value
        return checksum_value



#!/usr/bin/env python3

# Copyright (C) 2015  Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


import unittest

import os
import sys

DIR = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(DIR, ".."))

from productmd.common import is_valid_release_short, is_valid_release_version, parse_release_id, is_valid_release_type  # noqa
from productmd.common import split_version  # noqa
from productmd.common import create_release_id  # noqa
from productmd.common import get_major_version, get_minor_version  # noqa


class TestRelease(unittest.TestCase):

    def test_valid_short(self):
        self.assertTrue(is_valid_release_short("f"))
        self.assertFalse(is_valid_release_short("F"))

        self.assertTrue(is_valid_release_short("fedora"))
        self.assertFalse(is_valid_release_short("Fedora"))

        self.assertTrue(is_valid_release_short("fedora-server"))
        self.assertTrue(is_valid_release_short("fedora-server-23"))

        self.assertTrue(is_valid_release_short("f23"))
        self.assertFalse(is_valid_release_short("23f"))

        self.assertFalse(is_valid_release_short("-"))
        self.assertFalse(is_valid_release_short("f-"))
        self.assertTrue(is_valid_release_short("f-23"))
        self.assertFalse(is_valid_release_short("f--23"))
        self.assertFalse(is_valid_release_short("f.23"))

    def test_valid_version(self):
        self.assertTrue(is_valid_release_version("0"))
        self.assertTrue(is_valid_release_version("1"))

        self.assertTrue(is_valid_release_version("1.0"))
        self.assertTrue(is_valid_release_version("1.1"))

        self.assertTrue(is_valid_release_version("a"))
        self.assertFalse(is_valid_release_version("1.a"))
        self.assertFalse(is_valid_release_version("1.1a"))

        self.assertFalse(is_valid_release_version(""))

        self.assertFalse(is_valid_release_version("1."))
        self.assertFalse(is_valid_release_version("1.."))
        self.assertFalse(is_valid_release_version("1.1."))
        self.assertFalse(is_valid_release_version("1..1"))

        self.assertTrue(is_valid_release_version("rawhide"))

    def test_valid_release_type(self):
        self.assertTrue(is_valid_release_type("updates"))
        self.assertTrue(is_valid_release_type("updates-testing"))
        self.assertTrue(is_valid_release_type("e4s"))
        self.assertTrue(is_valid_release_type("e4s-testing"))

        self.assertFalse(is_valid_release_type(""))

        self.assertFalse(is_valid_release_type("-"))
        self.assertFalse(is_valid_release_type("-eus"))
        self.assertFalse(is_valid_release_type("4s"))

        self.assertFalse(is_valid_release_type("updates-"))

    def test_split_version(self):
        self.assertEqual(split_version("0"), [0])
        self.assertEqual(split_version("1.0"), [1, 0])

        self.assertEqual(split_version("rawhide"), ["rawhide"])
        self.assertEqual(split_version("rawhide.23"), ["rawhide.23"])

    def test_create_release_id(self):
        self.assertEqual(create_release_id("f", "23", "ga"), "f-23")
        self.assertEqual(create_release_id("f", "23", "updates"), "f-23-updates")
        self.assertRaises(TypeError, create_release_id, "f", "23", None)

    def test_parse_release_id(self):
        expected = {
            "short": "f",
            "version": "23",
            "type": "ga",
        }
        self.assertEqual(parse_release_id("f-23-ga"), expected)
        self.assertEqual(parse_release_id("f-23"), expected)

        expected = {
            "short": "f",
            "version": "23",
            "type": "updates",
        }
        self.assertEqual(parse_release_id("f-23-updates"), expected)

        expected = {
            "short": "f",
            "version": "23",
            "type": "updates-testing",
        }
        self.assertEqual(parse_release_id("f-23-updates-testing"), expected)


class TestGetMajorVersion(unittest.TestCase):
    def test_two_parts(self):
        self.assertEqual(get_major_version("1.0"), "1")

    def test_three_parts(self):
        self.assertEqual(get_major_version("1.0.0"), "1")

    def test_no_dots(self):
        self.assertEqual(get_major_version("Rawhide"), "Rawhide")


class TestGetMinorVersion(unittest.TestCase):
    def test_two_parts(self):
        self.assertEqual(get_minor_version("1.2"), "2")

    def test_three_parts(self):
        self.assertEqual(get_minor_version("1.2.3"), "2")

    def test_no_dots(self):
        self.assertEqual(get_minor_version("Rawhide"), None)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

# Copyright (C) 2015  Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


import unittest

import os
import sys

DIR = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(DIR, ".."))

from productmd.common import Header  # noqa


class TestHeader(unittest.TestCase):

    def test_version(self):
        hdr = Header(None, "productmd.header")

        # empty version
        hdr.version = None
        self.assertRaises(TypeError, hdr.validate)

        # invalid version
        hdr.version = "first"
        self.assertRaises(ValueError, hdr.validate)

        hdr.version = "1.alpha2"
        self.assertRaises(ValueError, hdr.validate)

        hdr.version = "1"
        self.assertRaises(ValueError, hdr.validate)

        # valid version
        hdr.version = "1.22"
        hdr.validate()

    def test_deserialize(self):
        hdr = Header(None, "productmd.header")
        data = {
            "header": {
                "type": "productmd.header",
                "version": "1.0",
            }
        }
        hdr.deserialize(data)
        self.assertEqual(hdr.version, "1.0")

    def test_serialize(self):
        hdr = Header(None, "productmd.header")
        hdr.version = "1.0"
        serialized_data = {}
        hdr.serialize(serialized_data)
        expected_data = {
            "header": {
                "type": "productmd.header",
                "version": "1.2",
            }
        }
        self.assertEqual(serialized_data, expected_data)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

# Copyright (C) 2015  Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


import unittest

import os
import sys
import tempfile
import shutil

DIR = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(DIR, ".."))

from productmd.treeinfo import TreeInfo, Variant  # noqa


class TestTreeInfo(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.treeinfo_path = os.path.join(DIR, "treeinfo")

    def setUp(self):
        self.maxDiff = None
        self.tmp_dir = tempfile.mkdtemp()
        self.ini_path = os.path.join(self.tmp_dir, "treeinfo")

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def assertSameFiles(self, path1, path2):
        self.assertEqual(os.path.getsize(path1), os.path.getsize(path2))
        with open(path1, "r") as file1:
            with open(path2, "r") as file2:
                self.assertEqual(file1.read(), file2.read())

    def _test_identity(self, ti):
        first = os.path.join(self.tmp_dir, "first")
        second = os.path.join(self.tmp_dir, "second")

        # write original file
        ti.dump(first)

        # read file and write it back
        ti = TreeInfo()
        ti.load(first)
        ti.dump(second)

        # check if first and second files are identical
        self.assertSameFiles(first, second)

    def test_create(self):
        ti = TreeInfo()
        ti.release.name = "Fedora"
        ti.release.short = "F"
        ti.release.version = "20"

        ti.tree.arch = "x86_64"
        ti.tree.build_timestamp = 123456

        variant = Variant(ti)
        variant.id = "Fedora"
        variant.uid = "Fedora"
        variant.name = "Fedora"
        variant.type = "variant"

        variant.paths.repository = "repo"
        variant.paths.packages = "pkgs"
        variant.paths.source_repository = "src repo"
        variant.paths.source_packages = "src pkgs"
        variant.paths.debug_repository = "debug repo"
        variant.paths.debug_packages = "debug pkgs"
        variant.paths.identity = "cert.pem"

        ti.variants.add(variant)

        ti.dump(self.ini_path)
        self._test_identity(ti)

    def test_create_with_two_variants(self):
        ti = TreeInfo()
        ti.release.name = "Fedora"
        ti.release.short = "F"
        ti.release.version = "20"

        ti.tree.arch = "x86_64"
        ti.tree.build_timestamp = 123456

        variant_1 = Variant(ti)
        variant_1.id = "AppStream"
        variant_1.uid = "AppStream"
        variant_1.name = "AppStream"
        variant_1.type = "variant"

        variant_1.paths.repository = "repo"
        variant_1.paths.packages = "pkgs"
        variant_1.paths.source_repository = "src repo"
        variant_1.paths.source_packages = "src pkgs"
        variant_1.paths.debug_repository = "debug repo"
        variant_1.paths.debug_packages = "debug pkgs"
        variant_1.paths.identity = "cert.pem"

        ti.variants.add(variant_1)

        variant_2 = Variant(ti)
        variant_2.id = "BaseOS"
        variant_2.uid = "BaseOS"
        variant_2.name = "BaseOS"
        variant_2.type = "variant"

        variant_2.paths.repository = "repo_2"
        variant_2.paths.packages = "pkgs_2"
        variant_2.paths.source_repository = "src repo 2"
        variant_2.paths.source_packages = "src pkgs 2"
        variant_2.paths.debug_repository = "debug repo 2"
        variant_2.paths.debug_packages = "debug pkgs 2"
        variant_2.paths.identity = "cert2.pem"

        ti.variants.add(variant_2)

        ti.dump(self.ini_path, main_variant="BaseOS")
        multivariant_ini_path = os.path.join(DIR, "treeinfo/multivariants_treeinfo")
        with open(self.ini_path, "r") as treeinfo_1:
            with open(multivariant_ini_path, "r") as treeinfo_2:
                self.assertEqual(treeinfo_1.read(), treeinfo_2.read())

    def test_f20(self):
        ti = TreeInfo()
        ti.load(os.path.join(DIR, "treeinfo/fedora-20-Fedora.x86_64"))

        # product
        self.assertEqual(ti.release.name, "Fedora")
        self.assertEqual(ti.release.short, "Fedora")
        self.assertEqual(ti.release.version, "20")

        # tree
        self.assertEqual(ti.tree.arch, "x86_64")
        # XXX: converts float to int
        self.assertEqual(ti.tree.build_timestamp, 1386857206)

        # variants
        self.assertEqual(len(ti.variants), 1)

        # variant: Fedora
        var = ti.variants["Fedora"]
        self.assertEqual(var.id, "Fedora")
        self.assertEqual(var.uid, "Fedora")
        self.assertEqual(var.name, "Fedora")
        self.assertEqual(var.type, "variant")

        self.assertEqual(var.paths.packages, "Packages")
        self.assertEqual(var.paths.repository, ".")

        # stage2
        self.assertEqual(ti.stage2.mainimage, "LiveOS/squashfs.img")
        self.assertEqual(ti.stage2.instimage, None)

        # images
        expected_images = {
            "kernel": "images/pxeboot/vmlinuz",
            "initrd": "images/pxeboot/initrd.img",
            "upgrade": "images/pxeboot/upgrade.img",
            "boot.iso": "images/boot.iso",
        }
        self.assertEqual(ti.images.images["x86_64"], expected_images)

        expected_images = {
            "kernel": "images/pxeboot/vmlinuz",
            "initrd": "images/pxeboot/initrd.img",
            "upgrade": "images/pxeboot/upgrade.img",
        }
        self.assertEqual(ti.images.images["xen"], expected_images)

        # checksums
        expected_checksums = {
            "images/boot.iso": ("sha256", "376be7d4855ad6281cb139430606a782fd6189dcb01d7b61448e915802cc350f"),
            "images/efiboot.img": ("sha256", "3bcba2a9b45366ab3f6d82dbe512421ddcb693f3bcbd9e30cc57aa0fa13c7835"),
            "images/macboot.img": ("sha256", "698b3492534399e31fa1033f51f25b85d463e743f6a26322a9b117f9aac4fdf3"),
            "images/pxeboot/initrd.img": ("sha256", "d0a81824e3425b6871ec4896a66e891aed35e291c50dfa30b08f6fc6ab04ca8b"),
            "images/pxeboot/upgrade.img": ("sha256", "a274b8756290447950f1e48cff9d9d6fd1144579ad98ed9f91929ff9ac1bbfa9"),
            "images/pxeboot/vmlinuz": ("sha256", "d3a2fbfcf08ac76dfb8135771380ec97ae3129b4e623891adb21bb1cd8ba59f6"),
            "repodata/repomd.xml": ("sha256", "108b4102829c0839c7712832577fe7da24f0a9491f4dc25d4145efe6aced2ebf"),
        }
        self.assertEqual(ti.checksums.checksums, expected_checksums)

        self._test_identity(ti)

    def test_f21_server(self):
        ti = TreeInfo()
        ti.load(os.path.join(DIR, "treeinfo/fedora-21-Alpha-Server.x86_64"))

        # product
        self.assertEqual(ti.release.name, "Fedora")
        self.assertEqual(ti.release.short, "Fedora")
        self.assertEqual(ti.release.version, "21")

        # tree
        self.assertEqual(ti.tree.arch, "x86_64")
        # XXX: converts float to int
        self.assertEqual(ti.tree.build_timestamp, 1410862874)

        # variants
        self.assertEqual(len(ti.variants), 1)

        # variant: Server
        var = ti.variants["Server"]
        self.assertEqual(var.id, "Server")
        self.assertEqual(var.uid, "Server")
        self.assertEqual(var.name, "Server")
        self.assertEqual(var.type, "variant")

        self.assertEqual(var.paths.packages, "Packages")
        self.assertEqual(var.paths.repository, ".")

        # stage2
        self.assertEqual(ti.stage2.mainimage, "LiveOS/squashfs.img")
        self.assertEqual(ti.stage2.instimage, None)

        # images
        expected_images = {
            "kernel": "images/pxeboot/vmlinuz",
            "initrd": "images/pxeboot/initrd.img",
            "upgrade": "images/pxeboot/upgrade.img",
            "boot.iso": "images/boot.iso",
        }
        self.assertEqual(ti.images.images["x86_64"], expected_images)

        expected_images = {
            "kernel": "images/pxeboot/vmlinuz",
            "initrd": "images/pxeboot/initrd.img",
            "upgrade": "images/pxeboot/upgrade.img",
        }
        self.assertEqual(ti.images.images["xen"], expected_images)

        # checksums
        expected_checksums = {
            "images/boot.iso": ("sha256", "e9c8b207d25c4af1a36c43a4cc3bf27dd2aefa9757eac63a8147e12e415f45f3"),
            "images/efiboot.img": ("sha256", "0655a92312272a2ee929f579c7ac5086c0ab75b7453a0293a38502d69a8335d3"),
            "images/macboot.img": ("sha256", "c25721c2e69ea82eedd78107baf69fc0d2900c5a23d0b689140edc2b372393f9"),
            "images/pxeboot/initrd.img": ("sha256", "b6cffe60149c0396b3413bf3f73c3a1aa3090048302daeeeb8830911d903aa2f"),
            "images/pxeboot/upgrade.img": ("sha256", "260ab3a63fb4ef1f0427a6b46d494c638760e0bb4e7e9f58295d0cf4e77886fc"),
            "images/pxeboot/vmlinuz": ("sha256", "974cbed883608046bacfc7a4f232320750150c2821edcd4650fbbe709a6c1d3d"),
            "repodata/repomd.xml": ("sha256", "1799b6652f287c73e1f1107ef5efe73a0714aaf34e8e9b81a08d3f741f5dc178"),
        }
        self.assertEqual(ti.checksums.checksums, expected_checksums)

        self._test_identity(ti)

    def test_read_treeinfo(self):
        for i in os.listdir(self.treeinfo_path):
            if i in ("fedora-8-Everything.x86_64", "fedora-8-Everything.ppc", "fedora-8-Everything.i386"):
                # version == "development" -> can't read correctly
                continue
            if "fedora" not in i:
                # Not a Fedora file
                continue
            path = os.path.join(self.treeinfo_path, i)
            ti = TreeInfo()
            ti.load(path)
            self.assertEqual(ti.release.short, "Fedora")
            self.assertEqual(ti.release.name, "Fedora")
            self.assertEqual(ti.release.version, str(int(ti.release.version)))

    def test_read_RHEL_treeinfo(self):
        for i in os.listdir(self.treeinfo_path):
            if "RHEL" not in i:
                continue
            if "optional" in i:
                continue
            path = os.path.join(self.treeinfo_path, i)
            ti = TreeInfo()
            ti.load(path)
            self.assertEqual(ti.release.short, "RHEL")
            self.assertEqual(ti.release.name, "Red Hat Enterprise Linux")
            self.assertEqual(ti.release.version, str(ti.release.version))

            # variants
            self.assertEqual(len(ti.variants), 2)

            # variant: BaseOS
            var = ti.variants["BaseOS"]
            self.assertEqual(var.id, "BaseOS")
            self.assertEqual(var.uid, "BaseOS")
            self.assertEqual(var.name, "BaseOS")
            self.assertEqual(var.type, "variant")

            self.assertEqual(var.paths.packages, "Packages")
            self.assertEqual(var.paths.repository, ".")

            # variant: AppStream
            var = ti.variants["AppStream"]
            self.assertEqual(var.id, "AppStream")
            self.assertEqual(var.uid, "AppStream")
            self.assertEqual(var.name, "AppStream")
            self.assertEqual(var.type, "variant")

            self.assertEqual(var.paths.packages, "Packages")

    def test_read_RHEL_Server_optional_treeinfo(self):
        path = os.path.join(self.treeinfo_path, "RHEL-7-Server-optional.x86_64")
        ti = TreeInfo()
        ti.load(path)

        var = ti.variants["Server-optional"]
        self.assertEqual(var.id, "optional")
        self.assertEqual(var.uid, "Server-optional")
        self.assertEqual(var.name, "optional")
        self.assertEqual(var.type, "optional")
        self.assertEqual(var.paths.packages, "Packages")
        self.assertEqual(var.paths.repository, ".")

    def test_treeinfo_compute_checksum(self):
        tmp_file = os.path.join(self.tmp_dir, "file")
        with open(tmp_file, "w") as f:
            f.write("test")
        expected_checksum = "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08"
        ti = TreeInfo()
        ti.checksums.add("file", "sha256", None, self.tmp_dir)
        self.assertEqual(ti.checksums.checksums["file"], ["sha256", expected_checksum])

    def test_parse_opensuse_treeinfo(self):
        ti = TreeInfo()
        ti.load(os.path.join(self.treeinfo_path, "opensuse"))
        self.assertEqual(ti.release.name, "openSUSE Leap")
        self.assertEqual(ti.release.short, "openSUSE Leap")
        self.assertEqual(ti.release.version, "15.1")
        self.assertEqual(ti.variants.variants, {})
        self.assertEqual(ti.tree.arch, "x86_64")
        self.assertEqual(ti.tree.platforms, set(["x86_64", "xen"]))
        self.assertEqual(
            ti.images.images,
            {
                "x86_64": {
                    "initrd": "boot/x86_64/loader/initrd",
                    "kernel": "boot/x86_64/loader/linux",
                },
                "xen": {
                    "initrd": "boot/x86_64/loader/initrd",
                    "kernel": "boot/x86_64/loader/linux",
                },
            },
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

# Copyright (C) 2015  Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


import unittest

import os
import sys
import tempfile
import shutil

DIR = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(DIR, ".."))

from productmd.images import Images, Image, identify_image  # noqa


class TestImages(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        self.tmp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def assertSameFiles(self, path1, path2):
        self.assertEqual(os.path.getsize(path1), os.path.getsize(path2))
        with open(path1, "r") as file1:
            with open(path2, "r") as file2:
                self.assertEqual(file1.read(), file2.read())

    def _test_identity(self, im):
        first = os.path.join(self.tmp_dir, "first")
        second = os.path.join(self.tmp_dir, "second")

        # write original file
        im.dump(first)

        # read file and write it back
        im = Images()
        im.load(first)
        im.dump(second)

        # check if first and second files are identical
        self.assertSameFiles(first, second)

    def test_fedora_20(self):
        im = Images()
        im.header.version = "1.0"
        im.compose.id = "Fedora-20-20131212.0"
        im.compose.type = "production"
        im.compose.date = "20131212"
        im.compose.respin = 0

        i = Image(im)
        i.path = "Fedora/x86_64/iso/Fedora-20-x86_64-DVD.iso"
        i.mtime = 1410855216
        i.size = 4603248640
        i.arch = "x86_64"
        i.type = "dvd"
        i.format = "iso"
        i.disc_number = 1
        i.disc_count = 1
        i.volume_id = "Fedora 20 x86_64"
        i.subvariant = ""

        # checksums
        i.add_checksum(root=None, checksum_type="sha256", checksum_value="f2eeed5102b8890e9e6f4b9053717fe73031e699c4b76dc7028749ab66e7f917")
        i.add_checksum(root=None, checksum_type="sha1", checksum_value="36dd25d7a6df45cdf19b85ad1bf2a2ccbf34f991")
        i.add_checksum(root=None, checksum_type="md5", checksum_value="9a190c8b2bd382c2d046dbc855cd2f2b")
        self.assertEqual(i.checksums, {
            "sha256": "f2eeed5102b8890e9e6f4b9053717fe73031e699c4b76dc7028749ab66e7f917",
            "sha1": "36dd25d7a6df45cdf19b85ad1bf2a2ccbf34f991",
            "md5": "9a190c8b2bd382c2d046dbc855cd2f2b",
        })
        self.assertRaises(ValueError, i.add_checksum, root=None, checksum_type="sha256", checksum_value="foo")

        i.implant_md5 = "b39b2f6770ca015f300af01cb54db75c"
        i.bootable = True
        im.add("Fedora", "x86_64", i)

        i = Image(im)
        i.path = "Fedora/x86_64/iso/Fedora-20-x86_64-netinst.iso"
        i.mtime = 1410855243
        i.size = 336592896
        i.arch = "x86_64"
        i.type = "netinst"
        i.format = "iso"
        i.disc_number = 1
        i.disc_count = 1
        i.volume_id = "Fedora 20 x86_64"
        i.subvariant = ""

        # checksums
        i.add_checksum(root=None, checksum_type="sha256", checksum_value="376be7d4855ad6281cb139430606a782fd6189dcb01d7b61448e915802cc350f")
        i.add_checksum(root=None, checksum_type="sha1", checksum_value="cb8b3e285fc1336cbbd7ba4b0381095dd0e159b0")
        i.add_checksum(root=None, checksum_type="md5", checksum_value="82716caf39ce9fd88e7cfc66ca219db8")
        self.assertEqual(i.checksums, {
            "sha256": "376be7d4855ad6281cb139430606a782fd6189dcb01d7b61448e915802cc350f",
            "sha1": "cb8b3e285fc1336cbbd7ba4b0381095dd0e159b0",
            "md5": "82716caf39ce9fd88e7cfc66ca219db8",
        })
        self.assertRaises(ValueError, i.add_checksum, root=None, checksum_type="sha256", checksum_value="foo")

        i.implant_md5 = "62cc05b03d28881c88ff1e949d6fc0b7"
        i.bootable = True
        im.add("Fedora", "x86_64", i)

        # im.dump("f20.json")

        self._test_identity(im)

        # Server: 1 arch, 2 images
        self.assertEqual(len(im["Fedora"]), 1)
        self.assertEqual(len(im["Fedora"]["x86_64"]), 2)

    def test_fedora_23(self):
        im = Images()
        im.header.version = "1.0"
        im.compose.id = "Fedora-23-20151030.0"
        im.compose.type = "production"
        im.compose.date = "20151030"
        im.compose.respin = 0

        i = Image(im)
        i.path = "Server/x86_64/iso/Fedora-Server-DVD-x86_64-23.iso"
        i.mtime = 1446169817
        i.size = 2149580800
        i.arch = "x86_64"
        i.type = "dvd"
        i.format = "iso"
        i.disc_number = 1
        i.disc_count = 1
        i.volume_id = "Fedora-S-23-x86_64"
        i.subvariant = ""

        # checksums
        i.add_checksum(root=None, checksum_type="sha256", checksum_value="30758dc821d1530de427c9e35212bd79b058bd4282e64b7b34ae1a40c87c05ae")
        self.assertEqual(i.checksums, {
            "sha256": "30758dc821d1530de427c9e35212bd79b058bd4282e64b7b34ae1a40c87c05ae",
        })
        self.assertRaises(ValueError, i.add_checksum, root=None, checksum_type="sha256", checksum_value="foo")

        i.implant_md5 = "1cd120922a791d03e829392a2b6b2107"
        i.bootable = True
        im.add("Server", "x86_64", i)

        i = Image(im)
        i.path = "Server/x86_64/iso/Fedora-Server-netinst-x86_64-23.iso"
        i.mtime = 1458057407
        i.size = 8011776
        i.arch = "x86_64"
        i.type = "netinst"
        i.format = "iso"
        i.disc_number = 1
        i.disc_count = 1
        i.volume_id = "Fedora-S-23-x86_64"
        i.subvariant = ""

        # checksums
        i.add_checksum(root=None, checksum_type="sha256", checksum_value="32e0a15a1c71d0e2fd36a0af5b67a3b3af82976d2dfca0aefcb90d42f2ae6844")
        self.assertEqual(i.checksums, {
            "sha256": "32e0a15a1c71d0e2fd36a0af5b67a3b3af82976d2dfca0aefcb90d42f2ae6844",
        })
        self.assertRaises(ValueError, i.add_checksum, root=None, checksum_type="sha256", checksum_value="foo")

        i.implant_md5 = "6ccc75afc55855ece24ee84e62e6dcc0"
        i.bootable = True
        im.add("Server", "x86_64", i)

        i = Image(im)
        i.path = "Live/x86_64/iso/Fedora-Live-KDE-x86_64-23-10.iso"
        i.mtime = 1446154932
        i.size = 1291845632
        i.arch = "x86_64"
        i.type = "live"
        i.format = "iso"
        i.disc_number = 1
        i.disc_count = 1
        i.volume_id = "Fedora-Live-KDE-x86_64-23-10"
        i.subvariant = "KDE"

        # checksums
        i.add_checksum(root=None, checksum_type="sha256", checksum_value="ef7e5ed9eee6dbcde1e0a4d69c76ce6fb552f75ccad879fa0f93031ceb950f27")
        self.assertEqual(i.checksums, {
            "sha256": "ef7e5ed9eee6dbcde1e0a4d69c76ce6fb552f75ccad879fa0f93031ceb950f27",
        })
        self.assertRaises(ValueError, i.add_checksum, root=None, checksum_type="sha256", checksum_value="foo")
        # identifier (Image instance)
        self.assertEqual(identify_image(i), ("KDE", "live", "iso", "x86_64", 1, False, []))
        # identifier (dict)
        parser = []
        i.serialize(parser)
        imgdict = parser[0]
        self.assertEqual(identify_image(imgdict), ("KDE", "live", "iso", "x86_64", 1, False, []))

        i.implant_md5 = "8bc179ecdd48e0b019365104f081a83e"
        i.bootable = True
        im.add("Live", "x86_64", i)

        # im.dump("f23.json")

        self._test_identity(im)

        # Server: 1 arch, 2 images
        self.assertEqual(len(im["Server"]), 1)
        self.assertEqual(len(im["Server"]["x86_64"]), 2)

        # Live: 1 arch, 1 images
        self.assertEqual(len(im["Live"]), 1)
        self.assertEqual(len(im["Live"]["x86_64"]), 1)

    def test_image_repr(self):
        i = Image(None)
        i.path = "Fedora/x86_64/iso/Fedora-20-x86_64-DVD.iso"
        i.mtime = 1410855216
        i.size = 4603248640
        i.arch = "x86_64"
        i.type = "dvd"
        i.format = "iso"
        i.disc_number = 1
        i.disc_count = 1
        i.volume_id = "Fedora 20 x86_64"

        self.assertEqual(
            repr(i),
            '<Image:Fedora/x86_64/iso/Fedora-20-x86_64-DVD.iso:iso:x86_64>')

    def test_image_repr_incomplete(self):
        i = Image(None)

        self.assertEqual(repr(i), '<Image:None:None:None>')

    def test_f20_format_10(self):
        im = Images()
        im.load(os.path.join(DIR, "images/f20.json"))

    def test_f23_format_11(self):
        im = Images()
        im.load(os.path.join(DIR, "images/f23.json"))

    def test_forbidden_src_arch(self):
        """
        Test: ValueError("Source arch is not allowed. Map source files under binary arches.")
        """
        im = Images()
        i = Image(im)
        self.assertRaises(ValueError, im.add, "Server", "src", i)

    def test_move_src_images_under_binary_arches(self):
        """
        Test if src images were moved under binary arches correctly.
        """
        before = os.path.join(DIR, "images/src_move_before.json")
        after = os.path.join(DIR, "images/src_move_after.json")
        converted = os.path.join(self.tmp_dir, "converted")

        im = Images()
        im.load(before)

        self._test_identity(im)

        im.dump(converted)
        self.assertSameFiles(converted, after)

    def test_unified_iso_serialized_only_with_true(self):
        i = Image(None)
        i.arch = 'x86_64'
        i.disc_count = 1
        i.disc_number = 1
        i.format = 'iso'
        i.type = 'dvd'
        i.mtime = 1410855216
        i.path = "Fedora/x86_64/iso/Fedora-20-x86_64-DVD.iso"
        i.size = 4603248640
        i.subvariant = 'Workstation'
        i.checksums = {'sha256': 'XXXXXX'}

        data = []
        i.serialize(data)
        self.assertFalse('unified' in data[0])

        i.unified = True
        data = []
        i.serialize(data)
        self.assertTrue(data[0]['unified'])

    def test_unified_iso_deserialize(self):
        im = Images()
        i = Image(im)

        data = {
            'arch': 'x86_64',
            'disc_count': 1,
            'disc_number': 1,
            'format': 'iso',
            'type': 'dvd',
            'mtime': 1410855216,
            'path': "Fedora/x86_64/iso/Fedora-20-x86_64-DVD.iso",
            'size': 4603248640,
            'subvariant': 'Workstation',
            'volume_id': None,
            'implant_md5': None,
            'bootable': True,
            'checksums': {'sha256': 'XXXXXX'},
        }

        i.deserialize(data)
        self.assertFalse(i.unified)

        data['unified'] = True
        i.deserialize(data)
        self.assertTrue(i.unified)

    def test_unique_id_enforcement(self):
        """Test that adding two images with different checksums but
        matching UNIQUE_IMAGE_ATTRIBUTES is disallowed (on 1.1+).
        """
        im = Images()
        im.header.version = '1.1'

        i1 = Image(im)
        i2 = Image(im)
        data = {
            'arch': 'x86_64',
            'disc_count': 1,
            'disc_number': 1,
            'format': 'iso',
            'type': 'dvd',
            'mtime': 1410855216,
            'path': "Fedora/x86_64/iso/Fedora-20-x86_64-DVD.iso",
            'size': 4603248640,
            'subvariant': 'Workstation',
            'volume_id': None,
            'implant_md5': None,
            'bootable': True,
        }
        # NOTE: there's a rather subtle behaviour here where when you
        # deserialize, mutable things in the deserialized object are
        # not *copies* of the objects in the dict you deserialized
        # but *are* those objects. So if you modify them in the dict
        # after deserialization, *the deserialized object changes*.
        # I'm not sure whether this is intentional, but it means we
        # must be careful here, we cannot just deserialize i1, change
        # the checksums in data, then deserialize i2; if we do that,
        # i1's checksums are changed, the checksums for i1 and i2
        # match, and the expected error isn't triggered.
        data1 = dict(data)
        data1['checksums'] = {'sha256': 'XXXXXX'}
        i1.deserialize(data1)
        im.add("Workstation", "x86_64", i1)

        data2 = dict(data)
        data2['checksums'] = {'sha256': 'YYYYYY'}
        i2.deserialize(data2)
        self.assertRaises(ValueError, im.add, "Server", "x86_64", i2)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

# Copyright (C) 2015  Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


import unittest

import os
import sys
import tempfile
import shutil

DIR = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(DIR, ".."))

from productmd.rpms import Rpms  # noqa


class TestRpms(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        self.tmp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def assertSameFiles(self, path1, path2):
        self.assertEqual(os.path.getsize(path1), os.path.getsize(path2))
        with open(path1, "r") as file1:
            with open(path2, "r") as file2:
                self.assertEqual(file1.read(), file2.read())

    def _test_identity(self, rm):
        first = os.path.join(self.tmp_dir, "first")
        second = os.path.join(self.tmp_dir, "second")

        # write original file
        rm.dump(first)

        # read file and write it back
        rm = Rpms()
        rm.load(first)
        rm.dump(second)

        # check if first and second files are identical
        self.assertSameFiles(first, second)

    def test_fedora_20(self):
        rm = Rpms()
        rm.header.version = "1.0"
        rm.compose.id = "Fedora-20-20131212.0"
        rm.compose.type = "production"
        rm.compose.date = "20131212"
        rm.compose.respin = 0

        # binary RPMs
        rm.add(
            "Fedora",
            "x86_64",
            "glibc-0:2.18-11.fc20.x86_64.rpm",
            path="Fedora/x86_64/os/Packages/g/glibc-2.18-11.fc20.x86_64.rpm",
            sigkey="246110c1",
            category="binary",
            srpm_nevra="glibc-0:2.18-11.fc20.src.rpm",
        )
        rm.add(
            "Fedora",
            "x86_64",
            "glibc-common-0:2.18-11.fc20.x86_64.rpm",
            path="Fedora/x86_64/os/Packages/g/glibc-common-2.18-11.fc20.x86_64.rpm",
            sigkey="246110c1",
            category="binary",
            srpm_nevra="glibc-0:2.18-11.fc20.src.rpm",
        )

        # source RPM
        rm.add(
            "Fedora",
            "x86_64",
            "glibc-0:2.18-11.fc20.src.rpm",
            path="Fedora/source/SRPMS/g/glibc-2.18-11.fc20.x86_64.rpm",
            sigkey="246110c1",
            category="source",
        )

        self._test_identity(rm)

        # 1 arch
        self.assertEqual(len(rm["Fedora"]), 1)

        # 1 SRPM
        self.assertEqual(len(rm["Fedora"]["x86_64"]), 1)

        # 3 RPMs (including 1 SRPM)
        self.assertEqual(len(rm["Fedora"]["x86_64"]["glibc-0:2.18-11.fc20.src"]), 3)

    def test_forbidden_src_arch(self):
        """
        Test: ValueError("Source arch is not allowed. Map source files under binary arches.")
        """
        rm = Rpms()
        self.assertRaises(
            ValueError,
            rm.add,
            "Fedora",
            "src",
            "glibc-0:2.18-11.fc20.src.rpm",
            path="Fedora/source/SRPMS/g/glibc-2.18-11.fc20.x86_64.rpm",
            sigkey="246110c1",
            category="source",
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

# Copyright (C) 2019  Red Hat, Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, see <https://gnu.org/licenses/>.


import unittest

import json
import os
import sys
import tempfile
import shutil

from io import StringIO

DIR = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(DIR, ".."))

from productmd.extra_files import ExtraFiles    # noqa


class TestExtraFiles(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        self.tmp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmp_dir, ignore_errors=True)

    def assertSameFiles(self, path1, path2):
        self.assertEqual(os.path.getsize(path1), os.path.getsize(path2))
        with open(path1, "r") as file1:
            with open(path2, "r") as file2:
                self.assertEqual(file1.read(), file2.read())

    def _test_identity(self, modules):
        first = os.path.join(self.tmp_dir, "first")
        second = os.path.join(self.tmp_dir, "second")

        # write original file
        modules.dump(first)

        # read file and write it back
        modules = ExtraFiles()
        modules.load(first)
        modules.dump(second)

        # check if first and second files are identical
        self.assertSameFiles(first, second)

    def test_bad_checksums(self):
        metadata = ExtraFiles()
        self.assertRaises(
            TypeError,
            metadata.add,
            "Everything",
            "x86_64",
            "path/to/file",
            size=1,
            checksums="no",
        )

    def test_bad_variant(self):
        metadata = ExtraFiles()
        self.assertRaises(
            ValueError, metadata.add, "", "x86_64", "path/to/file", size=1, checksums={}
        )

    def test_bad_arch(self):
        metadata = ExtraFiles()
        self.assertRaises(
            ValueError,
            metadata.add,
            "Everything",
            "foobar",
            "path/to/file",
            size=1,
            checksums={},
        )

    def test_bad_path(self):
        metadata = ExtraFiles()
        self.assertRaises(
            ValueError, metadata.add, "Everything", "foobar", "", size=1, checksums={}
        )

    def test_absolute_path(self):
        metadata = ExtraFiles()
        self.assertRaises(
            ValueError,
            metadata.add,
            "Everything",
            "foobar",
            "/path",
            size=1,
            checksums={},
        )

    def test_fedora_20(self):
        metadata = ExtraFiles()
        metadata.header.version = "1.0"
        metadata.compose.id = "Fedora-20-20131212.0"
        metadata.compose.type = "production"
        metadata.compose.date = "20131212"
        metadata.compose.respin = 0

        metadata.add(
            "Everything",
            "x86_64",
            "compose/Everything/x86_64/os/GPL",
            size=123,
            checksums={"md5": "abcde", "sha512": "a1b2c3"},
        )

        self._test_identity(metadata)

    def test_partial_dump(self):
        metadata = ExtraFiles()
        metadata.header.version = "1.0"
        metadata.compose.id = "Fedora-20-20131212.0"
        metadata.compose.type = "production"
        metadata.compose.date = "20131212"
        metadata.compose.respin = 0

        metadata.add(
            "Everything",
            "x86_64",
            "compose/Everything/x86_64/os/GPL",
            size=123,
            checksums={"md5": "abcde", "sha512": "a1b2c3"},
        )

        out = StringIO()
        metadata.dump_for_tree(out, "Everything", "x86_64", "compose/Everything/x86_64/os")
        self.assertEqual(
            json.loads(out.getvalue()),
            {
                "header": {"version": "1.0"},
                "data": [
                    {
                        "file": "GPL",
                        "size": 123,
                        "checksums": {"md5": "abcde", "sha512": "a1b2c3"},
                    },
                ],
            },
        )

    def test_partial_dump_in_deleted_directory(self):
        os.chdir(self.tmp_dir)
        shutil.rmtree(self.tmp_dir)

        metadata = ExtraFiles()
        metadata.header.version = "1.0"
        metadata.compose.id = "Fedora-20-20131212.0"
        metadata.compose.type = "production"
        metadata.compose.date = "20131212"
        metadata.compose.respin = 0

        metadata.add(
            "Everything",
            "x86_64",
            "compose/Everything/x86_64/os/GPL",
            size=123,
            checksums={"md5": "abcde", "sha512": "a1b2c3"},
        )

        out = StringIO()
        metadata.dump_for_tree(out, "Everything", "x86_64", "compose/Everything/x86_64/os")
